@@ -19,9 +19,9 @@
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py 2410 2007-05-21 18:28:50Z richb $"
-__version__   = "$Revision: 2410 $"
-__date__      = "$Date: 2007-05-21 14:28:50 -0400 (Mon, 21 May 2007) $"
+__id__        = "$Id: orca.py 2952 2007-10-01 20:01:08Z wwalker $"
+__version__   = "$Revision: 2952 $"
+__date__      = "$Date: 2007-10-01 16:01:08 -0400 (Mon, 01 Oct 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -47,8 +47,8 @@ except:
 import getopt
 import os
 import signal
-import string
 import time
+import unicodedata
 
 import atspi
 import braille
@@ -70,6 +70,14 @@ from input_event import InputEventHandler
 
 from orca_i18n import _           # for gettext support
 
+if settings.debugMemoryUsage:
+    import gc
+    gc.set_debug(gc.DEBUG_UNCOLLECTABLE
+                 | gc.DEBUG_COLLECTABLE
+                 | gc.DEBUG_INSTANCES
+                 | gc.DEBUG_OBJECTS
+                 | gc.DEBUG_SAVEALL)
+
 # The user-settings module (see loadUserSettings).
 #
 _userSettings = None
@@ -434,7 +442,8 @@ def _isPrintableKey(event_string):
     else:
         unicodeString = event_string.decode("UTF-8")
         reply = (len(unicodeString) == 1) \
-                and (unicodeString.isalnum() or unicodeString.isspace())
+                and (unicodeString.isalnum() or unicodeString.isspace()
+                or unicodedata.category(unicodeString)[0] in ('P', 'S'))
     debug.println(debug.LEVEL_FINEST,
                   "orca._echoPrintableKey: returning: %s" % reply)
     return reply
@@ -515,6 +524,30 @@ def _isActionKey(event_string):
                   "orca._echoActionKey: returning: %s" % reply)
     return reply
 
+class KeyEventType:
+    """Definition of available key event types."""
+
+    """An alphanumeric or punctuation key event."""
+    PRINTABLE = 'PRINTABLE'
+
+    """A modifier key event."""
+    MODIFIER = 'MODIFIER'
+
+    """A locking key event."""
+    LOCKING = 'LOCKING'
+
+    """A locking key lock event."""
+    LOCKING_LOCKED = 'LOCKING_LOCKED'
+
+    """A locking key unlock event."""
+    LOCKING_UNLOCKED = 'LOCKING_UNLOCKED'
+
+    """A function key event."""
+    FUNCTION = 'FUNCTION'
+
+    """An action key event."""
+    ACTION = 'ACTION'
+
 def _keyEcho(event):
     """If the keyEcho setting is enabled, check to see what type of key
     event it is and echo it via speech, if the user wants that type of
@@ -538,11 +571,6 @@ def _keyEcho(event):
     debug.println(debug.LEVEL_FINEST,
                   "orca._keyEcho: string to echo: %s" % event_string)
 
-    voices = settings.voices
-    voice = voices[settings.DEFAULT_VOICE]
-
-    lockState = None
-
     # If key echo is enabled, then check to see what type of key event
     # it is and echo it via speech, if the user wants that type of key
     # echoed.
@@ -552,24 +580,25 @@ def _keyEcho(event):
         if _isPrintableKey(event_string):
             if not settings.enablePrintableKeys:
                 return
-
-            if event_string.isupper():
-                voice = voices[settings.UPPERCASE_VOICE]
+            type = KeyEventType.PRINTABLE
 
         elif _isModifierKey(event_string):
             if not settings.enableModifierKeys:
                 return
+            type = KeyEventType.MODIFIER
 
         elif _isLockingKey(event_string):
             if not settings.enableLockingKeys:
                 return
+            type = KeyEventType.LOCKING
 
             modifiers = event.modifiers
+
             if event_string == "Caps_Lock":
                 if modifiers & (1 << atspi.Accessibility.MODIFIER_SHIFTLOCK):
-                    lockState = " " + _("off")
+                    type = KeyEventType.LOCKING_UNLOCKED
                 else:
-                    lockState = " " + _("on")
+                    type = KeyEventType.LOCKING_LOCKED
 
             elif event_string == "Num_Lock":
                 # [[[TODO: richb - we are not getting a correct modifier
@@ -578,32 +607,26 @@ def _keyEcho(event):
                 # until this can be fixed.]]]
                 #
                 #if modifiers & (1 << atspi.Accessibility.MODIFIER_NUMLOCK):
-                #    event_string += " " + _("off")
+                #    type = KeyEventType.LOCKING_UNLOCKED
                 #else:
-                #    event_string += " " + _("on")
+                #    type = KeyEventType.LOCKING_LOCKED
                 pass
 
         elif _isFunctionKey(event_string):
             if not settings.enableFunctionKeys:
                 return
+            type = KeyEventType.FUNCTION
 
         elif _isActionKey(event_string):
             if not settings.enableActionKeys:
                 return
+            type = KeyEventType.ACTION
 
         else:
             debug.println(debug.LEVEL_FINEST,
                   "orca._keyEcho: event string not handled: %s" % event_string)
             return
 
-        # Check to see if there are localized words to be spoken for
-        # this key event.
-        #
-        event_string = keynames.getKeyName(event_string)
-
-        if lockState:
-            event_string += lockState
-
         debug.println(debug.LEVEL_FINEST,
                       "orca._keyEcho: speaking: %s" % event_string)
 
@@ -612,7 +635,7 @@ def _keyEcho(event):
         #
         orca_state.lastKeyEchoTime = time.time()
 
-        speech.speak(event_string, voice)
+        speech.speakKeyEvent(event_string, type)
 
 def _processKeyCaptured(event):
     """Called when a new key event arrives and orca_state.capturingKeys=True.
@@ -727,7 +750,6 @@ def _processKeyboardEvent(event):
         keyboardEvent.modifiers = keyboardEvent.modifiers \
                                   | (1 << settings.MODIFIER_ORCA)
 
-
     # Orca gets first stab at the event.  Then, the presenter gets
     # a shot. [[[TODO: WDW - might want to let the presenter try first?
     # The main reason this is staying as is is that we may not want
@@ -766,7 +788,7 @@ def _processKeyboardEvent(event):
     orca_state.lastInputEvent = keyboardEvent
 
     # If this is a key event for a non-modifier key, save a handle to it.
-    # This is needed to help determine user actions when a multi-key chord 
+    # This is needed to help determine user actions when a multi-key chord
     # has been pressed, and we might get the key events in different orders.
     # See comment #15 of bug #435201 for more details.
     #
@@ -866,11 +888,15 @@ def loadUserSettings(script=None, inputEvent=None):
             reloaded = True
         except ImportError:
             debug.printException(debug.LEVEL_FINEST)
+        except:
+            debug.printException(debug.LEVEL_SEVERE)
     else:
         try:
             _userSettings = __import__("user-settings")
         except ImportError:
             debug.printException(debug.LEVEL_FINEST)
+        except:
+            debug.printException(debug.LEVEL_SEVERE)
 
     # If any settings were added to the command line, they take
     # precedence over everything else.
